@@ -612,6 +612,7 @@ Use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
             mn_onset = frac(mn_onset)
         except:
             self.logger.error(f"The mn_onset {mn_onset} could not be interpreted as a fraction.")
+            raise
         candidates = self.ml[self.ml['mn'] == mn]
         if len(candidates) == 0:
             self.logger.error(f"MN {mn} does not occur in measure list, which ends at MN {self.ml['mn'].max()}.")
@@ -662,12 +663,16 @@ The first ending MC {mc} is being used. Suppress this warning by using disambigu
         data['label_count'] = len(self.get_raw_labels())
         data['TimeSig'] = dict(self.ml.loc[self.ml.timesig != self.ml.timesig.shift(), ['mc', 'timesig']].itertuples(index=False, name=None))
         data['KeySig']  = dict(self.ml.loc[self.ml.keysig != self.ml.keysig.shift(), ['mc', 'keysig']].itertuples(index=False, name=None))
-        first_label =  self.soup.find('Harmony')
-        first_label_name = first_label.find('name') if first_label is not None else None
-        if first_label_name is not None and first_label_name.string is not None:
-            m = re.match(r"^\.?([A-Ga-g](#+|b+)?)", first_label_name.string)
-            if m is not None:
-                data['annotated_key'] = m.group(1)
+        annotated_key = None
+        for harmony_tag in self.soup.find_all('Harmony'):
+            label = harmony_tag.find('name')
+            if label is not None and label.string is not None:
+                m = re.match(r"^\.?([A-Ga-g](#+|b+)?)", label.string)
+                if m is not None:
+                    annotated_key = m.group(1)
+                    break
+        if annotated_key is not None:
+            data['annotated_key'] = annotated_key
         if len(self.nl.index) == 0:
             return data
         staff_groups = self.nl.groupby('staff').midi
