@@ -4,8 +4,9 @@ from tempfile import NamedTemporaryFile as Temp
 
 import pandas as pd
 
-from .utils import check_labels, color2rgba, convert, DCML_DOUBLE_REGEX, decode_harmonies, get_ms_version, get_musescore, no_collections_no_booleans,\
-    resolve_dir, rgba, rgba2attrs, rgba2params, test_binary, unpack_mscz, update_labels_cfg, update_cfg
+from .utils import check_labels, color2rgba, convert, DCML_DOUBLE_REGEX, decode_harmonies,\
+    get_ms_version, get_musescore, no_collections_no_booleans,\
+    resolve_dir, rgba2params, unpack_mscz, update_labels_cfg
 from .bs4_parser import _MSCX_bs4
 from .annotations import Annotations
 from .logger import LoggedClass
@@ -31,7 +32,7 @@ class Score(LoggedClass):
     RN_REGEX = r"^$"
     """:obj:`str`
     Class variable with a regular expression for Roman numerals that
-    romentarily matches nothing because ms3 tries interpreting Roman Numerals
+    momentarily matches nothing because ms3 tries interpreting Roman Numerals
     als DCML harmony annotations.
     """
 
@@ -45,6 +46,11 @@ class Score(LoggedClass):
     Formats that have to be converted before parsing.
     """
 
+    parseable_formats = native_formats + convertible_formats
+    """:obj:`tuple`
+    Formats that ms3 can parse.
+    """
+
     def __init__(self, musescore_file=None, infer_label_types=['dcml'], read_only=False, labels_cfg={}, logger_cfg={},
                  parser='bs4', ms=None):
         """
@@ -61,8 +67,8 @@ class Score(LoggedClass):
             Defaults to ``False``, meaning that the parsing is slower and uses more memory in order to allow for manipulations
             of the score, such as adding and deleting labels. Set to ``True`` if you're only extracting information.
         labels_cfg : :obj:`dict`
-            Store a configuration dictionary to determine the output format of the :py:class:`~ms3.annotations.Annotations`
-            object representing the currently attached annotations. See :obj:`MSCX.labels_cfg`.
+            Store a configuration dictionary to determine the output format of the :py:class:`~.annotations.Annotations`
+            object representing the currently attached annotations. See :py:attr:`MSCX.labels_cfg`.
         logger_cfg : :obj:`dict`, optional
             The following options are available:
             'name': LOGGER_NAME -> by default the logger name is based on the parsed file(s)
@@ -80,31 +86,31 @@ class Score(LoggedClass):
         self.full_paths = {}
         """:obj:`dict`
         ``{KEY: {i: full_path}}`` dictionary holding the full paths of all parsed MuseScore and TSV files,
-        including file names. Handled internally by :py:meth:`~ms3.score.Score._handle_path`.
+        including file names. Handled internally by :py:meth:`~.Score._handle_path`.
         """
 
         self.paths = {}
         """:obj:`dict`
         ``{KEY: {i: file path}}`` dictionary holding the paths of all parsed MuseScore and TSV files,
-        excluding file names. Handled internally by :py:meth:`~ms3.score.Score._handle_path`.
+        excluding file names. Handled internally by :py:meth:`~.Score._handle_path`.
         """
 
         self.files = {}
         """:obj:`dict`
         ``{KEY: {i: file name with extension}}`` dictionary holding the complete file name  of each parsed file,
-        including the extension. Handled internally by :py:meth:`~ms3.score.Score._handle_path`.
+        including the extension. Handled internally by :py:meth:`~.Score._handle_path`.
         """
 
         self.fnames = {}
         """:obj:`dict`
         ``{KEY: {i: file name without extension}}`` dictionary holding the file name  of each parsed file,
-        without its extension. Handled internally by :py:meth:`~ms3.score.Score._handle_path`.
+        without its extension. Handled internally by :py:meth:`~.Score._handle_path`.
         """
 
         self.fexts = {}
         """:obj:`dict`
         ``{KEY: {i: file extension}}`` dictionary holding the file extension of each parsed file.
-        Handled internally by :py:meth:`~ms3.score.Score._handle_path`.
+        Handled internally by :py:meth:`~.Score._handle_path`.
         """
 
         self.ms = get_musescore(ms, logger=self.logger)
@@ -118,14 +124,15 @@ class Score(LoggedClass):
 
         self._detached_annotations = {}
         """:obj:`dict`
-        ``{(key, i): Annotations object}`` dictionary for accessing all detached :py:class:`~ms3.annotations.Annotations` objects.            """
+        ``{(key, i): Annotations object}`` dictionary for accessing all detached :py:class:`~.annotations.Annotations` objects.
+        """
 
         self._types_to_infer = []
         """:obj:`list`
         Current order in which types are being recognized."""
 
         self._label_types = {
-            0: "Simple string (should not begin with a note name, otherwise MS3 will turn it into type 3; prevent through leading dot)",
+            0: "Simple string (does not begin with a note name, otherwise MS3 will turn it into type 3; prevent through leading dot)",
             1: "MuseScore's Roman Numeral Annotation format",
             2: "MuseScore's Nashville Number format",
             3: "Absolute chord encoded by MuseScore",
@@ -133,7 +140,7 @@ class Score(LoggedClass):
         }
         """:obj:`dict`
         Mapping label types to their descriptions.
-        0: "Simple string (should not begin with a note name, otherwise MS3 will turn it into type 3; prevent through leading dot)",
+        0: "Simple string (does not begin with a note name, otherwise MS3 will turn it into type 3; prevent through leading dot)",
         1: "MuseScore's Roman Numeral Annotation format",
         2: "MuseScore's Nashville Number format",
         3: "Absolute chord encoded by MuseScore",
@@ -161,13 +168,13 @@ class Score(LoggedClass):
             'positioning': True,
             'decode': False,
             'column_name': 'label',
-            'color_format': 'html',
+            'color_format': None,
         }
         """:obj:`dict`
-        Configuration dictionary to determine the output format of the :py:class:`~ms3.annotations.Annotations`
-        objects contained in :obj:`._annotations``. 
+        Configuration dictionary to determine the output format of the :py:class:`~.annotations.Annotations`
+        objects contained in the current object, especially when calling :py:attr:`Score.mscx.labels<.MSCX.labels>`.
         The default options correspond to the default parameters of
-        :py:meth:`Annotations.get_labels()<ms3.annotations.Annotations.get_labels>`.
+        :py:meth:`Annotations.get_labels()<.annotations.Annotations.get_labels>`.
         """
         self.labels_cfg.update(update_labels_cfg(labels_cfg, logger=self.logger))
 
@@ -178,7 +185,7 @@ class Score(LoggedClass):
 
         self.infer_label_types = infer_label_types
         if musescore_file is not None:
-            self._parse_mscx(musescore_file, read_only=read_only, labels_cfg=labels_cfg)
+            self._parse_mscx(musescore_file, read_only=read_only, labels_cfg=self.labels_cfg)
     #%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END of __init__() %%%%%%%%%%%%%%%%%%%%%%%%%%%%%#
     #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%#
 
@@ -186,7 +193,7 @@ class Score(LoggedClass):
     def infer_label_types(self):
         """:obj:`list` or :obj:`dict`, optional
         The order in which label types are to be inferred.
-        Assigning a new value results in a call to :py:meth:`~ms3.annotations.Annotations.infer_types`.
+        Assigning a new value results in a call to :py:meth:`~.annotations.Annotations.infer_types`.
         Passing a {label type: regex} dictionary is a shortcut to update type regex's or to add new ones.
         The inference will take place in the order in which they appear in the dictionary. To reuse an existing
         regex will updating others, you can refer to them as None, e.g. ``{'dcml': None, 'my_own': r'^(PAC|HC)$'}``.
@@ -222,7 +229,7 @@ class Score(LoggedClass):
     @property
     def has_detached_annotations(self):
         """:obj:`bool`
-        Is True as long as the score contains :py:class:`~ms3.annotations.Annotations` objects, that are not attached to the :obj:`MSCX` object.
+        Is True as long as the score contains :py:class:`~.annotations.Annotations` objects, that are not attached to the :obj:`MSCX` object.
         """
         return len(self._detached_annotations) > 0
 
@@ -241,7 +248,7 @@ class Score(LoggedClass):
         return self._label_types
 
 
-    def attach_labels(self, key, staff=None, voice=None, check_for_clashes=True, remove_detached=True):
+    def attach_labels(self, key, staff=None, voice=None, label_type=None, check_for_clashes=True, remove_detached=True):
         """ Insert detached labels ``key`` into this score's :obj:`MSCX` object.
 
         Parameters
@@ -250,11 +257,17 @@ class Score(LoggedClass):
             Key of the detached labels you want to insert into the score.
         staff, voice : :obj:`int`, optional
             Pass one or both of these arguments to change the original annotation layer or if there was none.
+        label_type : :obj:`int`, optional
+            By default, the labels are written into the staff's layer for absolute ('guitar') chords, meaning that when
+            opened next time, MuseScore will split and encode those beginning with a note name (internal label_type 3).
+            In order to influence how MuseScore treats the labels pass one of these values:
+            1: Roman Numeral Analysis
+            2: Nashville Numbers
         check_for_clashes : :obj:`bool`, optional
             Defaults to True, meaning that the positions where the labels will be inserted will be checked for existing
             labels.
         remove_detached : :obj:`bool`, optional
-            By default, the detached :py:class:`~ms3.annotations.Annotations` object is removed after successfully attaching it.
+            By default, the detached :py:class:`~.annotations.Annotations` object is removed after successfully attaching it.
             Pass False to have it remain in detached state.
 
         Returns
@@ -276,7 +289,7 @@ Use one of the existing keys or load a new set with the method load_annotations(
         if goal == 0:
             self.mscx.logger.warning(f"The Annotation object '{key}' does not contain any labels.")
             return 0, 0
-        df = annotations.prepare_for_attaching(staff=staff, voice=voice, check_for_clashes=check_for_clashes)
+        df = annotations.prepare_for_attaching(staff=staff, voice=voice, label_type=label_type, check_for_clashes=check_for_clashes)
         reached = len(df)
         if reached == 0:
             self.mscx.logger.error(f"No labels from '{key}' have been attached due to aforementioned errors.")
@@ -292,6 +305,26 @@ Use one of the existing keys or load a new set with the method load_annotations(
                 self.mscx.logger.info(f"Only {reached} of the {goal} targeted labels could be attached, so '{key}' was not removed.")
         return reached, goal
 
+    def change_labels_cfg(self, labels_cfg={}, staff=None, voice=None, label_type=None, positioning=None, decode=None,
+                          column_name=None, color_format=None):
+        """ Update :py:attr:`.Score.labels_cfg` and :py:attr:`.MSCX.labels_cfg`.
+
+        Parameters
+        ----------
+        labels_cfg : :obj:`dict`
+            Using an entire dictionary or, to change only particular options, choose from:
+        staff, voice, label_type, positioning, decode
+            Arguments as they will be passed to :py:meth:`~.annotations.Annotations.get_labels`
+        """
+        keys = ['staff', 'voice', 'label_type', 'positioning', 'decode', 'column_name', 'color_format']
+        for k in keys:
+            val = locals()[k]
+            if val is not None:
+                labels_cfg[k] = val
+        updated = update_labels_cfg(labels_cfg, logger=self.logger)
+        self.labels_cfg.update(updated)
+        self.mscx.labels_cfg.update(updated)
+
 
     def check_labels(self, keys='annotations', regex=None, label_type='dcml', **kwargs):
         """ Tries to match the labels ``keys`` against the given ``regex`` or the one of the registered ``label_type``.
@@ -308,7 +341,7 @@ Use one of the existing keys or load a new set with the method load_annotations(
             To use the regular expression of a registered type, pass its name, defaults to 'dcml'. Pass a new name and
             a ``regex`` to register a new label type on the fly.
         kwargs :
-            Parameters passed to :py:func:`~ms3.utils.check_labels`.
+            Parameters passed to :py:func:`~.utils.check_labels`.
 
         Returns
         -------
@@ -348,7 +381,7 @@ Use one of the existing keys or load a new set with the method load_annotations(
 
 
 
-    def compare_labels(self, detached_key, new_color='ms3_darkgreen', old_color='ms3_darkred', detached_is_newer=False):
+    def compare_labels(self, detached_key, new_color='ms3_darkgreen', old_color='ms3_darkred', detached_is_newer=False, add_to_rna=True):
         """ Compare detached labels ``key`` to the ones attached to the Score.
         By default, the attached labels are considered as the reviewed version and changes are colored in green;
         Changes with respect to the detached labels are attached to the Score in red.
@@ -362,6 +395,8 @@ Use one of the existing keys or load a new set with the method load_annotations(
         detached_is_newer : :obj:`bool`, optional
             Pass True if the detached labels are to be added with ``new_color`` whereas the attached changed labels
             will turn ``old_color``, as opposed to the default.
+        add_to_rna : :obj:`bool`, optional
+            By default, new labels are attached to the Roman Numeral layer. Pass false to attach them to the chord layer instead.
         """
         assert detached_key != 'annotations', "Pass a key of detached labels, not 'annotations'."
         if not self.mscx.has_annotations:
@@ -410,8 +445,16 @@ Use one of the existing keys or load a new set with the method load_annotations(
         df = pd.DataFrame(changes_old, columns=compare_cols)
         for k, v in added_color_params.items():
             df[k] = v
-        added_changes = self.mscx.add_labels(Annotations(df=df), )
-        if (added_changes is None or added_changes > 0) or color_changes > 0:
+        if add_to_rna:
+            df['label_type'] = 1
+            anno = Annotations(df=df)
+            anno.remove_initial_dots()
+        else:
+            df['label_type'] = 0
+            anno = Annotations(df=df)
+            anno.add_initial_dots()
+        added_changes = self.mscx.add_labels(anno)
+        if added_changes > 0 or color_changes > 0:
             self.mscx.changed = True
             self.mscx.parsed.parse_measures()
             self.mscx._update_annotations()
@@ -423,7 +466,7 @@ Use one of the existing keys or load a new set with the method load_annotations(
 
     def detach_labels(self, key, staff=None, voice=None, label_type=None, delete=True):
         """ Detach all annotations labels from this score's :obj:`MSCX` object or just a selection of them.
-        The extracted labels are stored as a new :py:class:`~ms3.annotations.Annotations` object that is accessible via ``Score.{key}``.
+        The extracted labels are stored as a new :py:class:`~.annotations.Annotations` object that is accessible via ``Score.{key}``.
         By default, ``delete`` is set to True, meaning that if you call :py:meth:`store_mscx` afterwards,
         the created MuseScore file will not contain the detached labels.
 
@@ -450,7 +493,7 @@ Use one of the existing keys or load a new set with the method load_annotations(
         if not self.mscx.has_annotations:
             self.mscx.logger.info("No annotations present in score.")
             return
-        assert key != 'annotations', "The key 'annotations' is reserved, please choose a different one."
+        assert key not in dir(self) + ['annotations'], f"The key {key} is reserved, please choose a different one."
         if not key.isidentifier():
             self.logger.warning(
                 f"'{key}' cannot be used as an identifier. The extracted labels need to be accessed via self._detached_annotations['{key}']")
@@ -469,6 +512,7 @@ Use one of the existing keys or load a new set with the method load_annotations(
         return
 
 
+
     def get_infer_regex(self):
         """
         Returns
@@ -493,7 +537,7 @@ Use one of the existing keys or load a new set with the method load_annotations(
         description : :obj:`str`, optional
             Human readable description that appears when calling the property ``Score.types``.
         infer : :obj:`bool`, optional
-            By default, the labels of all :py:class:`~ms3.annotations.Annotations` objects are matched against the new type.
+            By default, the labels of all :py:class:`~.annotations.Annotations` objects are matched against the new type.
             Pass False to not change any label's type.
         """
         assert name not in self._label_types, f"'{name}' already added to types: {self._label_types[name]}"
@@ -505,19 +549,19 @@ Use one of the existing keys or load a new set with the method load_annotations(
                 self[key].infer_types(self.get_infer_regex())
 
     def load_annotations(self, tsv_path=None, anno_obj=None, key='tsv', cols={}, infer=True):
-        """ Attach an :py:class:`~ms3.annotations.Annotations` object to the score and make it available as ``Score.{key}``.
+        """ Attach an :py:class:`~.annotations.Annotations` object to the score and make it available as ``Score.{key}``.
         It can be an existing object or one newly created from the TSV file ``tsv_path``.
 
         Parameters
         ----------
         tsv_path : :obj:`str`
-            If you want to create a new :py:class:`~ms3.annotations.Annotations` object from a TSV file, pass its path.
-        anno_obj : :py:class:`~ms3.annotations.Annotations`
+            If you want to create a new :py:class:`~.annotations.Annotations` object from a TSV file, pass its path.
+        anno_obj : :py:class:`~.annotations.Annotations`
             Instead, you can pass an existing object.
         key : :obj:`str`, defaults to 'tsv'
             Specify a new key for accessing the set of annotations. The string needs to be usable
             as an identifier, e.g. not start with a number, not contain special characters etc. In return you
-            may use it as a property: For example, passing ``'chords'`` lets you access the :py:class:`~ms3.annotations.Annotations` as
+            may use it as a property: For example, passing ``'chords'`` lets you access the :py:class:`~.annotations.Annotations` as
             ``Score.chords``. The key 'annotations' is reserved for all annotations attached to the score.
         cols : :obj:`dict`, optional
             If the columns in the specified TSV file diverge from the :ref:`standard column names<column_names>`,
@@ -548,7 +592,7 @@ Use one of the existing keys or load a new set with the method load_annotations(
         Parameters
         ----------
         key : :obj:`str`, optional
-            Key of the :py:class:`~ms3.annotations.Annotations` object which you want to output as TSV file.
+            Key of the :py:class:`~.annotations.Annotations` object which you want to output as TSV file.
             By default, the annotations attached to the score (key='annotations') are stored.
         tsv_path : :obj:`str`, optional
             Path of the newly created TSV file including the file name.
@@ -642,7 +686,7 @@ Use one of the existing keys or load a new set with the method load_annotations(
         and in the case of a compressed MuseScore file (.mscz), a temporary uncompressed file is generated
         which is removed after the parsing process.
         Essentially, parsing means to initiate a :obj:`MSCX` object and to make it available as ``Score.mscx``
-        and, if the score includes annotations, to initiate an :py:class:`~ms3.annotations.Annotations` object that
+        and, if the score includes annotations, to initiate an :py:class:`~.annotations.Annotations` object that
         can be accessed as ``Score.annotations``.
         The method doesn't systematically clean up data from a hypothetical previous parse.
 
@@ -656,8 +700,8 @@ Use one of the existing keys or load a new set with the method load_annotations(
         parser : 'bs4', optional
             The only XML parser currently implemented is BeautifulSoup 4.
         labels_cfg : :obj:`dict`, optional
-            Store a configuration dictionary to determine the output format of the :py:class:`~ms3.annotations.Annotations`
-            object representing the currently attached annotations. See :obj:`MSCX.labels_cfg`.
+            Store a configuration dictionary to determine the output format of the :py:class:`~.annotations.Annotations`
+            object representing the currently attached annotations. See :py:attr:`.MSCX.labels_cfg`.
         """
         if parser is not None:
             self.parser = parser
@@ -704,13 +748,25 @@ Use one of the existing keys or load a new set with the method load_annotations(
 
     def __repr__(self):
         msg = ''
-        if 'mscx' in self.full_paths:
-            msg = f"MuseScore file"
-            if self._mscx.changed:
-                msg += " (CHANGED!!!)\n---------------!!!!!!!!!!!!"
+        if any(ext in self.full_paths for ext in ('mscx', 'mscz')):
+            if 'mscx' in self.full_paths:
+                path = self.full_paths['mscx']
+                msg = f"Uncompressed MuseScore file"
             else:
-                msg += "\n--------------"
-            msg += f"\n\n{self.full_paths['mscx']}\n\n"
+                path = self.full_paths['mscz']
+                msg = f"ZIP compressed MuseScore file"
+            if self._mscx.changed:
+                msg += " (CHANGED!!!)"
+        else:
+            frst = list(self.full_paths.keys())[0]
+            path = self.full_paths[frst]
+            msg = f"Temporarily converted {frst.upper()} file"
+        n_chars = len(msg)
+        if self._mscx.changed:
+            msg += '\n' + (n_chars-12) * '-' + 12 * '!'
+        else:
+            msg += '\n' + n_chars * '-'
+        msg += f"\n\n{path}\n\n"
         if self.mscx.has_annotations:
             msg += f"Attached annotations\n--------------------\n\n{self.annotations}\n\n"
         else:
@@ -720,6 +776,8 @@ Use one of the existing keys or load a new set with the method load_annotations(
             for key, obj in self._detached_annotations.items():
                 key_info = key + f" (stored as {self.files[key]})" if key in self.files else key
                 msg += f"{key_info} -> {obj}\n\n"
+        if self.mscx.n_form_labels > 0:
+            msg += f"Score contains {self.mscx.n_form_labels} form labels."
         return msg
 
 
@@ -782,8 +840,8 @@ class MSCX(LoggedClass):
         parser : :obj:`str`, optional
             Which XML parser to use.
         labels_cfg : :obj:`dict`, optional
-            Store a configuration dictionary to determine the output format of the :py:class:`~ms3.annotations.Annotations`
-            object representing the currently attached annotations. See :obj:`MSCX.labels_cfg`.
+            Store a configuration dictionary to determine the output format of the :py:class:`~.annotations.Annotations`
+            object representing the currently attached annotations. See :py:attr:`.MSCX.labels_cfg`.
         logger_cfg : :obj:`dict`, optional
             The following options are available:
             'name': LOGGER_NAME -> by default the logger name is based on the parsed file(s)
@@ -818,7 +876,7 @@ class MSCX(LoggedClass):
         of the score, such as adding and deleting labels. Set to ``True`` if you're only extracting information."""
 
         self._annotations = None
-        """:py:class:`~ms3.annotations.Annotations` or None
+        """:py:class:`~.annotations.Annotations` or None
         If the score contains at least one <Harmony> tag, this attribute points to the object representing all
         annotations, otherwise it is None."""
 
@@ -837,10 +895,10 @@ class MSCX(LoggedClass):
 
         self.labels_cfg = labels_cfg
         """:obj:`dict`
-        Configuration dictionary to determine the output format of the :py:class:`~ms3.annotations.Annotations`
-        object representing the labels that are attached to a score (stored as :obj:`._annotations``). 
+        Configuration dictionary to determine the output format of the :py:class:`~.annotations.Annotations`
+        object representing the labels that are attached to a score (stored as :py:attr:`._annotations``). 
         The options correspond to the parameters of
-        :py:meth:`Annotations.get_labels()<ms3.annotations.Annotations.get_labels>`.
+        :py:meth:`Annotations.get_labels()<.annotations.Annotations.get_labels>`.
         """
 
         ms_version = get_ms_version(self.mscx_src)
@@ -860,6 +918,15 @@ use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
     #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%#
 
 
+    @property
+    def cadences(self):
+        """:obj:`pandas.DataFrame`
+        DataFrame representing all cadence annotations in the score.
+        """
+        exp = self.expanded
+        if exp is None or 'cadence' not in exp.columns:
+            return None
+        return exp[exp.cadence.notna()]
 
 
     @property
@@ -870,7 +937,7 @@ use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
         all kinds of score markup that is not attached to particular notes but to a <Chord>, such as
         slurs, lyrics, staff text, ottava lines etc.
         """
-        return self._parsed.chords
+        return self.parsed.chords
 
 
     @property
@@ -880,7 +947,7 @@ use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
         i.e. <Chord>, <Rest>, <Harmony> and markup tags such as <Beam> together with, in the columns the values of their
         XML properties and children. It serves as master for computing :obj:`.chords`, :obj:`rests`, and :obj:`labels`
         (and therefore :obj:`.expanded`, too)."""
-        return self._parsed.events
+        return self.parsed.events
 
 
     @property
@@ -889,9 +956,9 @@ use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
         DataFrame of labels that have been split into various features using a regular expression."""
         if self._annotations is None:
             return None
-        labels_cfg = self.labels_cfg.copy()
+        #labels_cfg = self.labels_cfg.copy()
         #labels_cfg['decode'] = False
-        return self._annotations.expand_dcml(**labels_cfg)
+        return self._annotations.expand_dcml(**self.labels_cfg)
 
 
     @property
@@ -906,10 +973,27 @@ use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
         self.parsed.has_annotations = val
 
 
+    @property
+    def n_form_labels(self):
+        """:obj:`int`
+        Shortcut for ``MSCX.parsed.n_form_labels``.
+        Is True if at least one StaffText seems to constitute a form label."""
+        return self.parsed.n_form_labels
+
+
+    @property
+    def form_labels(self):
+        """:obj:`pandas.DataFrame`
+        DataFrame representing a filtered event list containing only StaffTexts that include the regular expression
+        :py:const:`.utils.FORM_DETECTION_REGEX`
+        """
+        return self.parsed.fl
+
+
     @property
     def labels(self):
         """:obj:`pandas.DataFrame`
-        DataFrame representing all <Harmony> tags in the score as returned by calling :py:meth:`~ms3.annotations.Annotations.get_labels`
+        DataFrame representing all <Harmony> tags in the score as returned by calling :py:meth:`~.annotations.Annotations.get_labels`
         on the object at :obj:`._annotations` with the current :obj:`._labels_cfg`."""
         if self._annotations is None:
             self.logger.info("The score does not contain any annotations.")
@@ -926,7 +1010,7 @@ use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
         (measure count). The columns represent for every MC its :ref:`actual duration<act_dur>`, its
         :ref:`time signature<timesig>`, how it is to be considered when computing measure numbers (:ref:`mn<mn>`),
         and which other MCs can "come :ref:`next`" according to the score's repeat structure."""
-        return self._parsed.ml
+        return self.parsed.ml
 
 
     @property
@@ -941,19 +1025,19 @@ use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
     def notes(self):
         """:obj:`pandas.DataFrame`
         DataFrame representing all <Note> tags within the score."""
-        return self._parsed.nl
+        return self.parsed.nl
 
 
     @property
     def notes_and_rests(self):
         """:obj:`pandas.DataFrame`
         The union of :obj:`.notes` and :obj:`.rests`."""
-        return self._parsed.notes_and_rests
+        return self.parsed.notes_and_rests
 
 
     @property
     def parsed(self):
-        """{:obj:`_MSCX_bs4`}
+        """:obj:`~._MSCX_bs4`
         Standard way of accessing the object exposed by the current parser. :obj:`MSCX` uses this object's
         interface for requesting manipulations of and information from the source XML."""
         if self._parsed is None:
@@ -966,36 +1050,43 @@ use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
     def rests(self):
         """:obj:`pandas.DataFrame`
         DataFrame representing all <Rest> tags."""
-        return self._parsed.rl
+        return self.parsed.rl
 
 
     @property
     def staff_ids(self):
         """:obj:`list` of :obj:`int`
         The staff IDs contained in the score, usually just a list of increasing numbers starting at 1."""
-        return self._parsed.staff_ids
+        return self.parsed.staff_ids
+
+    @property
+    def style(self):
+        """:obj:`Style`
+        Can be used like a dictionary to change the information within the score's <Style> tag."""
+        return self.parsed.style
 
 
     @property
     def version(self):
         """:obj:`str`
         MuseScore version that the file was created with."""
-        return self._parsed.version
+        return self.parsed.version
+
+    @property
+    def volta_structure(self):
+        """:obj:`dict`
+        {first_mc -> {volta_number -> [mc1, mc2...]} } dictionary."""
+        return self.parsed.volta_structure
 
 
     def add_labels(self, annotations_object):
-        """ Receives the labels from an :py:class:`~ms3.annotations.Annotations` object and adds them to the XML structure
+        """ Receives the labels from an :py:class:`~.annotations.Annotations` object and adds them to the XML structure
         representing the MuseScore file that might be written to a file afterwards.
 
         Parameters
         ----------
-        df : :obj:`pandas.DataFrame`
-            DataFrame with labels to be added.
-        columns : :obj:`dict`
-            If your columns don't have standard names, pass a {NAME -> ACTUAL_NAME} dictionary.
-            Required columns: label, mc, mc_onset, staff, voice
-            Additional columns: label_type, root, base, leftParen, rightParen, offset_x, offset_y, nashville, color_name,
-            color_html, color_r, color_g, color_b, color_a
+        annotations_object : :py:class:`~.annotations.Annotations`
+            Object of labels to be added.
 
         Returns
         -------
@@ -1006,14 +1097,14 @@ use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
         df = annotations_object.df
         if len(df) == 0:
             self.logger.info("Nothing to add.")
-            return
+            return 0
         main_cols = Annotations.main_cols
         columns = annotations_object.cols
         missing_main = {c for  c in main_cols if columns[c] not in df.columns}
-        assert len(
-            missing_main) == 0, f"The specified columns for the following main parameters are missing:\n{missing_main}"
+        assert len(missing_main) == 0, f"The specified columns for the following main parameters are missing:\n{missing_main}"
         if columns['decoded'] not in df.columns:
             df[columns['decoded']] = decode_harmonies(df, label_col=columns['label'], return_series=True)
+        #df = df[df[columns['label']].notna()]
         existing_cols = {k: v for k, v in columns.items() if v in df.columns}
         param2cols = {**existing_cols}
         parameters = list(param2cols.keys())
@@ -1061,21 +1152,24 @@ use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
                                               color_name=color_name, color_html=color_html, color_r=color_r,
                                               color_g=color_g, color_b=color_b, color_a=color_a)
 
-    def change_labels_cfg(self, labels_cfg={}, staff=None, voice=None, label_type=None, positioning=None, decode=None, color_format=None):
-        """ Update :obj:`MSCX.labels_cfg`.
+    def change_labels_cfg(self, labels_cfg={}, staff=None, voice=None, label_type=None, positioning=None, decode=None,
+                          column_name=None, color_format=None):
+        """ Update :py:attr:`.MSCX.labels_cfg`.
 
         Parameters
         ----------
         labels_cfg : :obj:`dict`
             Using an entire dictionary or, to change only particular options, choose from:
         staff, voice, label_type, positioning, decode
-            Arguments as they will be passed to :py:meth:`~ms3.annotations.Annotations.get_labels`
+            Arguments as they will be passed to :py:meth:`~.annotations.Annotations.get_labels`
         """
-        for k in self.labels_cfg.keys():
+        keys = ['staff', 'voice', 'label_type', 'positioning', 'decode', 'column_name', 'color_format']
+        for k in keys:
             val = locals()[k]
             if val is not None:
                 labels_cfg[k] = val
-        self.labels_cfg.update(update_labels_cfg(labels_cfg), logger=self.logger)
+        updated = update_labels_cfg(labels_cfg, logger=self.logger)
+        self.labels_cfg.update(updated)
 
 
 
@@ -1105,6 +1199,22 @@ use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
                 self.logger.warning(f"{target - changes} labels could not be deleted:\n{df.loc[~changed]}")
 
 
+    def replace_labels(self, annotations_object):
+        """
+
+        Parameters
+        ----------
+        annotations_object : :py:class:`~.annotations.Annotations`
+            Object of labels to be added.
+
+        Returns
+        -------
+
+        """
+        self.delete_labels(annotations_object.df)
+        self.add_labels(annotations_object)
+
+
 
     def delete_empty_labels(self):
         """ Remove all empty labels from the attached annotations. """
@@ -1175,7 +1285,7 @@ use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
     def get_raw_labels(self):
         """Shortcut for ``MSCX.parsed.get_raw_labels()``.
         Retrieve a "raw" list of labels, meaning that label types reflect only those defined within <Harmony> tags
-        which can be 1 (Nashville), 2 (MuseScore's Roman Numeral display) or undefined (in the case of 'normal'
+        which can be 1 (MuseScore's Roman Numeral display), 2 (Nashville) or undefined (in the case of 'normal'
         chord labels, defaulting to 0).
 
         Returns
@@ -1254,7 +1364,7 @@ use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
         Parameters
         ----------
         what : :obj:`str` or :obj:`Collection`, optional
-            Defaults to 'all' but could instead be one or several strings out of {'notes', 'rests', 'notes_and_rests', 'measures', 'events', 'labels', 'chords', 'expanded'}
+            Defaults to 'all' but could instead be one or several strings out of {'notes', 'rests', 'notes_and_rests', 'measures', 'events', 'labels', 'chords', 'expanded', 'form_labels'}
         folder : :obj:`str`, optional
             Where to store. Defaults to the directory of the parsed MSCX file.
         suffix : :obj:`str` or :obj:`Collection`, optional
@@ -1277,7 +1387,7 @@ use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
             folder = mscx_path
         if not os.path.isdir(folder):
             if input(folder + ' does not exist. Create? (y|n)') == "y":
-                os.mkdir(d)
+                os.makedirs(folder)
             else:
                 return
         what, suffix = self._treat_storing_params(what, suffix)
@@ -1302,7 +1412,7 @@ use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
                 self.logger.debug(f"{w} empty, no file written.")
 
     def _treat_storing_params(self, what, suffix):
-        tables = ['notes', 'rests', 'notes_and_rests', 'measures', 'events', 'labels', 'chords', 'expanded']
+        tables = ['notes', 'rests', 'notes_and_rests', 'measures', 'events', 'labels', 'chords', 'expanded', 'form_labels']
         if what == 'all':
             if suffix is None:
                 return tables, [f"_{t}" for t in tables]
